from math import *
from sprite import *

import pygame

TILESIZE = 32

class Actor(object):
	""" Actor class. """
	def __init__( self, x, y, level, color, gravity_mod=0 ):
		"""
		Actor class

		x is the starting x
		y is the starting y
		gravity_mod is a constant multiplier to the gravity of the object. It
		affects the gravity of all owners of this object
		"""
		self.x = x
		self.y = y
		self.vx = 0
		self.vy = 0
		self.level = level
		self.gravity_mod = gravity_mod
		self.item = None
		self.color = color
		self.has_key = False
	
	def contains( self, x, y):
		myrect = pygame.Rect((self.x, self.y), (TILESIZE, TILESIZE))
		return myrect.collidepoint(x, y)

	def isbutton(self):
		return False

	def update_location( self ):
		"""
		Updates the location of the actor with what should be the new location
		based on the actors vx, and vy.
		"""
		newlocations = self.level.gravity.get_new_location(self)
		
		myrectx = pygame.Rect((newlocations[0], self.y), (TILESIZE, TILESIZE))
		myrecty = pygame.Rect((self.x, newlocations[2]), (TILESIZE, TILESIZE))
		
		for actor in self.level.actors:
			rect = pygame.Rect((actor.x, actor.y), (TILESIZE, TILESIZE))
			if myrectx.colliderect(rect):
				if actor.isbutton():
					actor.press()
				newlocations = (self.x, 0, newlocations[2], newlocations[3])
				myrectx = pygame.Rect((newlocations[0], self.y), (TILESIZE, TILESIZE))
			if myrecty.colliderect(rect):
				if actor.isbutton():
					actor.press()
				newlocations = (newlocations[0], newlocations[1], self.y, 0)
				myrecty = pygame.Rect((self.x, newlocations[2]), (TILESIZE, TILESIZE))
		
		for door in self.level.doors:
			rect = pygame.Rect((door.x, door.y), (TILESIZE, TILESIZE))
			if myrectx.colliderect(rect):
				if door == self.level.finaldoor:
					self.level = self.level.finaldoor.open()
					self.item = None
					self.has_key = False
					return
				if self.has_key:
					self.level.doors.remove(door)
				else:
					newlocations = (self.x, 0, newlocations[2], newlocations[3])
					myrectx = pygame.Rect((newlocations[0], self.y), (TILESIZE, TILESIZE))
			if myrecty.colliderect(rect):
				if door == self.level.finaldoor:
					self.level = self.level.finaldoor.open()
					self.item = None
					self.has_key = False
					return
				if self.has_key:
					self.level.doors.remove(door)
				else:
					newlocations = (newlocations[0], newlocations[1], self.y, 0)
					myrecty = pygame.Rect((self.x, newlocations[2]), (TILESIZE, TILESIZE))
		
		for wall in self.level.walls:
			rect = pygame.Rect((wall.x, wall.y), (TILESIZE - 16, TILESIZE - 16))
			if myrectx.colliderect(rect):
				newlocations = (self.x, 0, newlocations[2], newlocations[3])
				myrectx = pygame.Rect((newlocations[0], self.y), (TILESIZE, TILESIZE))
			if myrecty.colliderect(rect):
				newlocations = (newlocations[0], newlocations[1], self.y, 0)
				myrecty = pygame.Rect((self.x, newlocations[2]), (TILESIZE, TILESIZE))
		
		for button in self.level.buttons:
			rect = pygame.Rect((button.x, button.y), (TILESIZE, TILESIZE))
			if myrectx.colliderect(rect):
				button.press()
				newlocations = (self.x, 0, newlocations[2], newlocations[3])
				myrectx = pygame.Rect((newlocations[0], self.y), (TILESIZE, TILESIZE))
			if myrecty.colliderect(rect):
				button.press()
				newlocations = (newlocations[0], newlocations[1], self.y, 0)
				myrecty = pygame.Rect((self.x, newlocations[2]), (TILESIZE, TILESIZE))
		
		(self.x, self.vx, self.y, self.vy) = newlocations
	
	def render(self, images, screen):
		if self.color == "Red":
			screen.blit(images[Sprite.RED_BLOCK], (self.x, self.y, TILESIZE, TILESIZE))
		elif self.color == "Yellow":
			screen.blit(images[Sprite.YELLOW_BLOCK], (self.x, self.y, TILESIZE, TILESIZE))
		elif self.color == "Blue":
			screen.blit(images[Sprite.BLUE_BLOCK], (self.x, self.y, TILESIZE, TILESIZE))
			
	def iskey(self):
		return False
