import pygame
import cPickle as pickle
import re
import physics.gravity_functions, physics.gravity
from actor import *
from button import *
from buttondoor import *
from wall import *
from key import *
from keydoor import *
from sprite import *
from finaldoor import *

TILESIZE = 32

class LevelRenderer(object):
	""" Level Renderer """
	levelnumber = 1
	def __init__( self, images ):
		self.images = images
	
	def loadLevel( self, filename ):
		file = open(filename, 'rb')
		self.level = pickle.load(file)
		file.close()
		self.level.levelloader = self
		
	def render( self, screen ):
		for obj in self.level.doors + self.level.walls + self.level.actors + self.level.buttons:
			obj.render(self.images, screen)
	
#	def play(self):
#		self.start_time = pygame.time.get_ticks()
#		self.player_is_dead = True
#		
#		
#		
#		self.end_time = pygame.time.get_ticks()
#		
#		return (self.player_is_dead, self.end_time - self.start_time)

class Level(object):
	""" Level class """
	"""
		level.start = start position (tuple)
		level.doors = list of tuples of door locations?
	"""
	def __init__( self, filename, gravity = None):
		self.levelloader = None
		self.tiles = []
		self.gravity = physics.gravity.Gravity(gravity)
		self.start = None
		self.doors = []
		self.actors = []
		self.walls = []
		self.buttons = []
		self.redbtn = None
		self.reddoor = None
		self.yellowbtn = None
		self.yellowdoor = None
		self.bluebtn = None
		self.bluedoor = None
		self.key = None
		self.finaldoor = None
		
		file = open(filename, 'r')
		y = 0
		while 1:
			line = file.readline()
			if line.strip() == 'ENDMAP':
				break
			
			self.tiles.append(line);
			for x in xrange(0, self.width()):
				c = self.tile(x, y)
				if c == '#':
					self.walls.append(Wall(x * TILESIZE, y * TILESIZE, self))
				if c == 'K':
					self.key = Key(x * TILESIZE, y * TILESIZE, self)
					self.actors.append(self.key)
				if c == 'L':
					self.doors.append(KeyDoor(x * TILESIZE, y * TILESIZE, self))
				if c == 'A':
					self.actors.append(Actor(x * TILESIZE, y * TILESIZE, self, "Red", -100))
				if c == 'B':
					self.bluebtn = Button(x * TILESIZE, y * TILESIZE, self, "Blue")
					self.buttons.append(self.bluebtn)
				if c == 'b':
					self.bluedoor = ButtonDoor(x * TILESIZE, y * TILESIZE, self, "Blue")
					self.doors.append(self.bluedoor)
				if c == 'C':
					self.actors.append(Actor(x * TILESIZE, y * TILESIZE, self, "Yellow"))
				if c == 'D':
					self.actors.append(Actor(x * TILESIZE, y * TILESIZE, self, "Blue", 100))
				if c == 'E':
					self.redbtn = Button(x * TILESIZE, y * TILESIZE, self, "Red")
					self.buttons.append(self.redbtn)
				if c == 'e':
					self.reddoor = ButtonDoor(x * TILESIZE, y * TILESIZE, self, "Red")
					self.doors.append(self.reddoor)
				if c == 'F':
					self.yellowbtn = Button(x * TILESIZE, y * TILESIZE, self, "Yellow")
					self.buttons.append(self.yellowbtn)
				if c == 'f':
					self.yellowdoor = ButtonDoor(x * TILESIZE, y * TILESIZE, self, "Yellow")
					self.doors.append(self.yellowdoor)
				if c == 'X':
					self.finaldoor = FinalDoor(x * TILESIZE, y * TILESIZE, self)
					self.doors.append(self.finaldoor)
			y = y + 1
		
		if self.redbtn is not None: self.redbtn.door = self.reddoor
		if self.yellowbtn is not None: self.yellowbtn.door = self.yellowdoor
		if self.bluebtn is not None: self.bluebtn.door = self.bluedoor
		
		while 1:
			line = file.readline()
			line = line.strip()
			if line == 'ENDLEVEL':
				break

			m = re.search(r"([a-z]+) = (.+)", line)
			if m.group(1) == "start":
				m_sub = re.search(r"\(([0-9]+), ([0-9]+)\)", m.group(2))
				self.start = (int(m_sub.group(1)), int(m_sub.group(2)))
		
		file.close()
			
	def width( self ):
		return len(self.tiles[0])
		
	def height( self ):
		return len(self.tiles)
		
	def tile( self, x, y ):
		return self.tiles[y][x]
		
	def isBlocked( self, x, y ):
		if self.tile(x, y) == '#':
			return True
		return False
		
	def pickle( self, filename ):
		file = open(filename, 'wb')
		pickle.dump(self, file)
		file.close()
	
	def open(self, door):
		if door is not None and door in self.doors:
			self.doors.remove(door)

if __name__ == "__main__":
	l = Level("levels/Level1_src.txt", physics.gravity_functions.constant_gravity(196))
	l.pickle("levels/Level1.txt")
	l = Level("levels/Level2_src.txt", physics.gravity_functions.array_gravity([ x/10.0 for x in range(-1960, 1960)], 1024))
	l.pickle("levels/Level2.txt")
	l = Level("levels/Level3_src.txt", physics.gravity_functions.array_gravity(range(-100,100) + range(100,-100,2) + range(-100, 100), 1024))
	l.pickle("levels/Level3.txt")
	l = Level("levels/Level4_src.txt", physics.gravity_functions.dual_gravity(range(-100,100), \
																			   range(100,-100), 1024))
	l.pickle("levels/Level4.txt")
