TIMESTEP = 0.016

class Gravity(object):
	""" Gravity Class """
	def __init__( self, grav_function ):
		"""
		Gravity class, takes a grav_function which should take two parameters,
		x and y, and return the gravitational constant at that point
		"""
		self.grav_function = grav_function
	

	def get_new_location( self, actor ):
		"""
		Gets the next location for the actor.

		Actor should at least have an x and y property and vx and vy. This will
		take into account the possibility that an actor may have an item that
		has a gravity_mod

		returns a tuple (x, vx, y, vy)
		"""
		grav_constant = self.grav_function( actor.x, actor.y )
		grav_constant += self._gravity_mod(actor)

		vx = actor.vx
		x = actor.x + vx * TIMESTEP

		vy = actor.vy + TIMESTEP * grav_constant
		y = actor.y + vy * TIMESTEP + 0.5 * grav_constant * TIMESTEP**2

		return (x, vx, y, vy)
	
	def _gravity_mod( self, actor ):
		"""
		Gets the accumulated gravity for actor and all its items
		"""
		if actor is None:
			return 1
		if actor.gravity_mod is not None:
			return actor.gravity_mod + self._gravity_mod(actor.item)
		else:
			return self._gravity_mod(actor.item)
