"""
Contains a basic set of helper gravity functions
"""

from math import *

class constant_gravity( object ):
	""" The obvious, returns a constant gravity """
	def __init__( self, gravity ):
		self.gravity = gravity
	
	def __call__( self, x, y ):
		return self.gravity

class array_gravity( object ):
	"""
	Returns a callable gravity where the gravity is looked up linearly in
	the array based on width

	>>> f = array_gravity([0, 1, 2, 3, 4], 10)
	>>> f(0, 0)
	0
	>>> f(1, 0)
	0
	>>> f(2, 0)
	1
	>>> f(3, 0)
	1
	>>> f(4, 0)
	2
	>>> f(5, 0)
	2
	>>> f(6, 0)
	3
	>>> f(7, 0)
	3
	>>> f(8, 0)
	4
	>>> f(9, 0)
	4
	"""
	def __init__( self, arr, width ):
		self.arr = arr
		self.width = width
	
	def __call__( self, x, y ):
		index = floor((len(self.arr) * x)/self.width)
		return self.arr[int(index)]


class dual_gravity( object ):
	"""
	Returns a callable gravity where the gravity is looked up linearly in
	the array based on width

	>>> f = array_gravity([0, 1, 2, 3, 4], 10)
	>>> f(0, 0)
	0
	>>> f(1, 0)
	0
	>>> f(2, 0)
	1
	>>> f(3, 0)
	1
	>>> f(4, 0)
	2
	>>> f(5, 0)
	2
	>>> f(6, 0)
	3
	>>> f(7, 0)
	3
	>>> f(8, 0)
	4
	>>> f(9, 0)
	4
	"""
	def __init__( self, arr, arr2, width ):
		self.arr = arr
		self.arr2 = arr2
		self.width = width
	
	def __call__( self, x, y ):
		index = floor((len(self.arr) * x)/self.width)
		if y >= 768/2:
			value = self.arr[int(index)]
		else:
			value = -self.arr[int(index)]
		return value
def _test():
	import doctest
	doctest.testmod()

if __name__ == "__main__":
	_test()
