import actor
from level import TILESIZE
from sprite import *

class Player(actor.Actor):
	"""
	Protaganist

	has self.facing_right, no self.facing_left for DRYability
	"""
	def __init__( self, startx, starty, level ):
		super(Player, self).__init__(startx, starty, level, None)
		self.facing_right = True
		self.has_key = False
		self.item = None
		
	def drop_item(self):
		if self.item == None:
			return
		
		self.has_key = False
		
		if self.facing_right:
			self.item.x = self.x + TILESIZE
			self.item.y = self.y
			item = self.item
			self.item = None
			return item
		else:
			self.item.x = self.x - TILESIZE
			self.item.y = self.y
			item = self.item
			self.item = None
			return item

	def pickup_item( self ):
		"""
		Attempt to pick up the item in front of the player. Will do nothing if
		the player is already carrying an item
		"""
		#Already carrying an item, can't carry more
		if self.item != None:
			return

		# get objects we're facing
		if self.facing_right:
			objs = [ actor for actor in self.level.actors if
					actor.contains(self.x+TILESIZE+2, self.y + TILESIZE/2) ]
		else:
			objs = [ actor for actor in self.level.actors if
					actor.contains(self.x-TILESIZE+2, self.y + TILESIZE/2) ]

		# not facing an object
		if len(objs) == 0:
			return

		self.item = objs[0]
		if self.item.iskey():
			self.has_key = True
		return self.item
	
	def unlock_door( self ):
		"""
		Attempt to unlock the door in front of the player

		return True if a door was unlocked, returns true and unlocked the door
		otherwise
		"""
		if self._has_key:
			return False

		if self.facing_right:
			doors = [ door for door in self.level.doors if
					door.in_location(x+TILESIZE, y + TILESIZE/2) ]
		else:
			doors = [ door for door in self.level.doors if
					door.in_location(x-TILESIZE, y + TILESIZE/2) ]

		if len(doors) == 0:
			return False

		door[0].open = True
		self._has_key = False
	
	def render( self, images, screen):
		if self.vy == 0:
			if self.facing_right:
				screen.blit(images[Sprite.PROTAGONIST_RIGHT], (self.x, self.y, TILESIZE, TILESIZE))
			else:
				screen.blit(images[Sprite.PROTAGONIST_LEFT], (self.x, self.y, TILESIZE, TILESIZE))
		else:
			if self.facing_right:
				screen.blit(images[Sprite.PROTAGONIST_JUMP_RIGHT], (self.x, self.y, TILESIZE, TILESIZE))
			else:
				screen.blit(images[Sprite.PROTAGONIST_JUMP_LEFT], (self.x, self.y, TILESIZE, TILESIZE))

